Glydo.Statusbar = Prototype.Class.create({
	initialize: function(application) {
		this.application = application;
		this.localizedStrings = new Glydo.Utils.LocalizedBundle(document);
		document.loadBindingDocument("chrome://glydo/content/ui/recommendations/views/recommendation_views.xml");
		this.categoryGroups = [];
		this.statusbarBox = document.getElementById('glydo-statusbar-box');
		this.categoryButtonsBox = document.getElementById('glydo-statusbar-category-buttons-box');
		this.statusbarHover = false;
		this.fixedButtonsBox = document.getElementById('glydo-statusbar-fixed-buttons-box');
		this.mainButton = document.getElementById('glydo-statusbar-main-button');
		this.homeButton = document.getElementById('glydo-statusbar-home-button');
		if ("_X_has_home_button_X_") {
			this.homeButton.hidden = false;
			this.homeButton.addEventListener("click",function() {
				Glydo.WindowUtils.goToURL(window,'_X_home_button_target_url_X_','current');
			},false);
		}
		this.initTicker();
		Glydo.Utils.addListener(this.statusbarBox,"mouseover",Prototype.F.curry(this.onStatusbarHover,true),this,false);
		Glydo.Utils.addListener(this.statusbarBox,"mouseout",Prototype.F.curry(this.onStatusbarHover,false),this,false);
		this.updateMainButton();
		this.buildCategoriesMap();
		Glydo.Prefs.on_ticker_change_interval_millis_changed(Prototype.F.bind(this.onPrefChangeTickerChangeInterval,this));
	},

	initTicker: function() {
		this.tickerState = null;
		this.tickerTimer = null;
		this.tickerFrozen = false;
		this.tickerBox = document.getElementById("glydo-statusbar-ticker-box");
		this.ticker = document.getElementById("glydo-statusbar-ticker");
		this.tickerPrevButton = document.getElementById("glydo-statusbar-ticker-prev");
		this.tickerNextButton = document.getElementById("glydo-statusbar-ticker-next");
		Glydo.Utils.addListener(this.tickerPrevButton,"click",this.tickerPrevUser,this,false);
		Glydo.Utils.addListener(this.tickerNextButton,"click",this.tickerNextUser,this,false);
	},
	
	buildCategoriesMap: function() {
		this.categoriesMap = {};
		for (var i = 0; i < Glydo.Statusbar.CATEGORIES.length; ++i) {
			var group = Glydo.Statusbar.CATEGORIES[i];
			for (var j = 0; j < group.categories.length; ++j) {
				var category = group.categories[j];
				this.categoriesMap[category.name] = category;
				if (!category.dontShowCategoryButton) {
					category.title = this.localizedStrings.get("vertical_pane." + category.name + ".title");
					if (category.adFilter === undefined) {
						category.adFilter = Glydo.Statusbar.PANE_GENERAL_ADS_FILTER;
					}
				}
			}
		}
	},
	
	onDocumentManagerStateChange: function() {
		this.updateMainButton();
	},
	
	updateMainButton: function() {
		if (this.application.documentManager.isProcessing()) {
			this.mainButton.className = "glydo-status-processing";
			return;
		}
		if (!this.application.documentManager.containsValidDocuments()) {
			this.mainButton.className = ""; 
			return;
		}
		this.mainButton.className = "glydo-status-normal";
	},
	
	update: function(docEntry) {
		this.clear();
		if (!docEntry || !docEntry.isDone()) {
			return;
		}
		this.updateCategories(docEntry);
		this.updateTicker(docEntry);
		docEntry.markRendered();
	},

	createCategory: function(docEntry,def) {
		var categoryName = def.name;
		var recs = docEntry.selectRecommendations(def.filter);
		if (!recs || recs.length === 0) {
			return null;
		}
		return ({
			categoryDef: def,
			recs: recs
		});
	},

	createCategoriesGroup: function(docEntry,groupDef) {
		var categories = [];
		if (docEntry.hasRecommendations()) {
			for (var i = 0; i < groupDef.categories.length; ++i) {
				var categoryDef = groupDef.categories[i];
				var category = this.createCategory(docEntry,categoryDef);
				if (!category) {
					continue;
				}
				if (!categoryDef.dontShowCategoryButton) {
					var categoryBox = document.createElement("hbox");
					categoryBox.className = "glydo-statusbar-category-box";
					categoryBox.id = "glydo-statusbar-category-box-" + categoryDef.name;
					categoryBox.setAttribute("align","center");
					
					var categoryButton = document.createElement("image");
					categoryButton.setAttribute("tooltiptext", categoryDef.title);
					categoryButton.className = "glydo-statusbar-category-button";
					categoryButton.id = "glydo-statusbar-category-button-"+categoryDef.name;
					categoryBox.appendChild(categoryButton);
					category.box = categoryBox;
					category.button = categoryButton;
					Glydo.Utils.addListener(categoryButton,"click",Prototype.F.curry(this.onCategoryClick,category),this,false);
				}
		
				categories.push(category);
			}
		}
		return ({
			groupDef: groupDef,
			categories: categories
		});
	},
	
	updateCategories: function(docEntry) {
		// Get categories state
		var categoryGroups = docEntry.getViewSpecificData("statusbar_categoryGroups");
		if (!categoryGroups) {
			categoryGroups = [];
			for (var g = 0; g < Glydo.Statusbar.CATEGORIES.length; ++g) {
				var group = this.createCategoriesGroup(docEntry,Glydo.Statusbar.CATEGORIES[g]);
				if (group) {
					categoryGroups.push(group);
				}
			}

			docEntry.setViewSpecificData("statusbar_categoryGroups",categoryGroups);
		}
		this.categoryGroups = categoryGroups;
		
		// Render categories
		categoryGroups.forEach(function(group) {
			group.categories.forEach(function(category) {
				if (category.box) {
					this.categoryButtonsBox.appendChild(category.box);
				}
			},this);
		},this);
	},

	updateTicker: function(docEntry) {
		if (docEntry) {
			// Get ticker state
			var tickerState = docEntry.getViewSpecificData("statusbar_ticker_state");
			if (!tickerState) {
				var groupsPositions = [];
				for (var g = 0; g < this.categoryGroups.length; ++g) {
					groupsPositions.push({
						curCategory: null,
						curRecPerCategory: []
					});
				}
				tickerState = ({
					tickerSteps: 0,
					groupsPositions: groupsPositions
				});
				docEntry.setViewSpecificData("statusbar_ticker_state",tickerState);
			}
			this.tickerState = tickerState;
		}
		// Render ticker
		if (Glydo.Prefs.ticker_show) {
			this.startTicker();
		}
	},
	
	clear: function(event) {
		this.clearTicker();
		this.clearCategories();
	},

	clearCategories: function() {
		this.categoryGroups.forEach(function(categoryGroup) {
			categoryGroup.categories.forEach(function(category) {
				if (category.box) {
					this.categoryButtonsBox.removeChild(category.box);
				}
				if (category.button) {
					Prototype.E.removeClassName(category.button,"glydo-category-open");
				}
			},this);
		},this);
		this.categoryGroups = [];
	},
	
	clearTicker: function() {
		this.unrenderTicker();
		this.tickerState = null;
		if (this.tickerTimer !== null) {
			clearInterval(this.tickerTimer);
			this.tickerTimer = null;
		}
	},
	
	updateTickerFreezeState: function() {
		if (this.application.detachedPopupsManager.isPopupOpen() || this.statusbarHover) {
			this.freezeTicker();
		} else {
			this.unfreezeTicker();
		}
	},
	
	onCategoryClick: function(category,event) {
		if (event.button !== 0) {
			return;
		}
		var categoryDef = category.categoryDef;
//		if (event.ctrlKey) {
//			this.application.detachedPopupsManager.toggleDetachedWindowAndHidePopup(categoryDef,{
//				anchor: categoryButton
//			});
//		} else {
//			this.application.detachedPopupsManager.focusWindowOrTogglePopup(categoryDef,{
//				anchor: categoryButton
//			});
//		}
		var opened = this.application.detachedPopupsManager.focusWindowOrTogglePopup(categoryDef,{
			anchor: category.button,
			ensureContainedWidth: categoryDef.expectedWidth,
			realEstateKind: "STATUSBAR_POPUP"
		});
		Glydo.LOGGING_DB.logUserEvent("statusbar.buttons.category",
				opened ? "open" : "close",
				{ mode: "normal", name: categoryDef.name }, null);
	},

	findCategoryByName: function(name) {
		var ng = this.categoryGroups.length;
		for (var g = 0; g < ng; ++g) {
			var group = this.categoryGroups[g];
			var nc = group.categories.length;
			for (var i = 0; i < nc; ++i) {
				var c = group.categories[i];
				if (c.categoryDef.name == name) {
					return c;
				}
			}
		}
	},
	
	onPopupOpening: function(name) {
		var c = this.findCategoryByName(name);
		if (c) {
			if (c.button) {
				Prototype.E.addClassName(c.button,"glydo-category-open");
			}
		}
	},

	onPopupOpened: function(name) {
		this.updateTickerFreezeState();
	},
	
	onPopupClosed: function(name,reason) {
		this.updateTickerFreezeState();
		var c = this.findCategoryByName(name);
		if (c) {
			if (c.button) {
				Prototype.E.removeClassName(c.button,"glydo-category-open");
			}
		}
	},

	onCurrentProcessedDocumentChanged: function(docEntry) {
		this.update(docEntry);
	},
	
	countRecsInGroup: function(group) {
		var l = group.categories.length;
		var n = 0;
		for (var i = 0; i < l; ++i) {
			n += group.categories[i].recs.length;
		}
		return n;
	},
	
	startTicker: function() {
		// Group 0 is always the regular group
		var n = this.countRecsInGroup(this.categoryGroups[0]);
		if (n > 0) {
			// If there is just one regular recommendation, we don't show ads or the ticker buttons
			this.tickerPrevButton.hidden = (n <= 1);
			this.tickerNextButton.hidden = (n <= 1);
			this.tickerAdsPeriod = (n <= 1) ? null : Glydo.Prefs.ticker_ads_period; 
			this.tickerNext();
			this.setupTickerTimer();
		}
	},
	
	setupTickerTimer: function() {
		if (this.tickerTimer) {
			clearInterval(this.tickerTimer);
			this.tickerTimer = null;
		}
		if (Glydo.Prefs.ticker_show) {
			this.tickerTimer = setInterval(Prototype.F.bind(this.tickerNext,this), Glydo.Prefs.ticker_change_interval_millis);
		}
	},
	
	clearTickerTimer: function() {
		if (this.tickerTimer) {
			clearInterval(this.tickerTimer);
		}
	},
	
	freezeTicker: function() {
		this.clearTickerTimer();
		this.tickerFrozen = true;
	},
	
	unfreezeTicker: function() {
		if (Glydo.Prefs.ticker_show) {
			this.setupTickerTimer();
		}
		this.tickerFrozen = false;
	},

	determineTickerRecGroup: function() {
		var groupIndex = 0;
		if (this.tickerState && this.tickerAdsPeriod) {
			var r = this.tickerState.tickerSteps % this.tickerAdsPeriod;
			if ((r >= 0 && r < 1) || (r<0 && (r+this.tickerAdsPeriod<1))) {
				groupIndex = 1;
			}
		}
		if (!this.categoryGroups[groupIndex] || !this.categoryGroups[groupIndex].categories || this.categoryGroups[groupIndex].categories.length == 0) {
			groupIndex = 0;
		}
		return groupIndex;
	},
	
	tickerNext: function() {
		// First, unrender the previous ticker if it exists
		this.unrenderTicker();
		
		if (this.categoryGroups.length <= 0 || !this.tickerState) {
			return;
		}
		
		++this.tickerState.tickerSteps;
		var groupIndex = this.determineTickerRecGroup();
		var position = this.tickerState.groupsPositions[groupIndex];
		var categories = this.categoryGroups[groupIndex].categories;

		if (categories.length == 0) {
			return;
		}
		
		if (position.curCategory === null) {
			position.curCategory = -1;
		}
		
		++position.curCategory;
		if (position.curCategory >= categories.length) {
			position.curCategory = 0;
		}

		var curCategory = categories[position.curCategory];
		var nextTickerRecInCategory = position.curRecPerCategory[position.curCategory];
		if (nextTickerRecInCategory === undefined) {
			nextTickerRecInCategory = -1;
		}
		++nextTickerRecInCategory;
		if (nextTickerRecInCategory >= curCategory.recs.length) {
			nextTickerRecInCategory = 0;
		}
		position.curRecPerCategory[position.curCategory] = nextTickerRecInCategory;
		this.renderTicker();
	},

	tickerPrev: function() {
		// First, unrender the previous ticker if it exists
		this.unrenderTicker();
		
		if (this.categoryGroups.length <= 0 || !this.tickerState) {
			return;
		}
		
		var groupIndex = this.determineTickerRecGroup();
		var position = this.tickerState.groupsPositions[groupIndex];
		var categories = this.categoryGroups[groupIndex].categories;

		if (position.curCategory === null) {
			position.curCategory = 0;
		}
		
		var curCategory = categories[position.curCategory];
		var prevTickerRecInCategory = position.curRecPerCategory[position.curCategory];
		if (prevTickerRecInCategory === undefined) {
			prevTickerRecInCategory = 0;
		}
		--prevTickerRecInCategory;
		if (prevTickerRecInCategory < 0) {
			prevTickerRecInCategory = curCategory.recs.length-1;
		}
		position.curRecPerCategory[position.curCategory] = prevTickerRecInCategory;

		// Go back one category
		--position.curCategory;
		if (position.curCategory < 0) {
			position.curCategory = categories.length - 1;
		}

		// Go back one ticker step and find appropriate position and categories
		--this.tickerState.tickerSteps;
		groupIndex = this.determineTickerRecGroup();
		position = this.tickerState.groupsPositions[groupIndex];
		categories = this.categoryGroups[groupIndex].categories;
		
		if (categories.length > 0) {
			// Make sure the current category has a rec set
			if (position.curRecPerCategory[position.curCategory] === undefined) {
				position.curRecPerCategory[position.curCategory] = categories[position.curCategory].recs.length-1;
			}
			this.renderTicker();
		}
	},

	tickerPrevUser: function() {
		Glydo.LOGGING_DB.logUserEvent("statusbar.ticker.prev","click",null,this.tickerState.tickerSteps);
		this.tickerPrev();
	},
	
	tickerNextUser: function() {
		Glydo.LOGGING_DB.logUserEvent("statusbar.ticker.next","click",null,this.tickerState.tickerSteps);
		this.tickerNext();
	},
	
	getCurrentTickerRecommendation: function() {
		var groupIndex = this.determineTickerRecGroup();
		var position = this.tickerState.groupsPositions[groupIndex];
		var categories = this.categoryGroups[groupIndex].categories;
		var curRec = null;
		var curCategory = null;
		if (position && position.curCategory !== null) {
			curCategory = categories[position.curCategory];
			if (curCategory && position.curRecPerCategory[position.curCategory] !== undefined) {
				curRec = curCategory.recs[position.curRecPerCategory[position.curCategory]];
			}
		}
		if (curRec) {
			return {rec: curRec, category: curCategory};
		}
		return null;
	},
	
	renderTicker: function() {
		var cur = this.getCurrentTickerRecommendation();
		if (!cur) {
			return;
		}
		var curRec = cur.rec;
		var curCategory = cur.category;
		if (curCategory && curCategory.button) {
			Prototype.E.addClassName(curCategory.button,"glydo-statusbar-category-emphasized");
		}
		if (!curRec) {
			this.tickerBox.hidden = true;
			return;
		} else {
			this.tickerBox.hidden = false;
			this.ticker.setAttribute("realEstatePosition","steps[" + this.tickerState.tickerSteps + "]");
			// Next, update the ticker contents
			this.curTickerClassName = Glydo.RECOMMENDATION_VIEW_CLASSES_MAP.lookup(curRec);
			Prototype.E.addClassName(this.ticker,this.curTickerClassName);
			// For some reason this forces the binding to be attached
			Glydo.Utils.getClientWidth(this.ticker);
			this.ticker.rec = curRec;
		}
	},

	unrenderTicker: function() {
		if (this.tickerState) {
			var cur = this.getCurrentTickerRecommendation();
			if (cur && cur.category && cur.category.button) {
				Prototype.E.removeClassName(cur.category.button,"glydo-statusbar-category-emphasized");
			}
		}
		this.tickerBox.hidden = true;
		this.ticker.rec = null;
		if (this.curTickerClassName) {
			Prototype.E.removeClassName(this.ticker,this.curTickerClassName);
		}
		this.curTickerClassName = null;
	},
	
	onPrefChangeTickerChangeInterval: function() {
		if (this.tickerTimer && Glydo.Prefs.ticker_show) {
			// There was a ticker already and we still have to show it - only update the timer
			this.setupTickerTimer();
			return;
		}
		// Otherwise we either need to hide a visible ticker, or show a hidden ticker - we call update
		this.updateTicker();
	},
	
	onStatusbarHover: function(enter,event) {
		if (enter) {
			if (this.statusbarHover) {
				return;
			}
		} else {
			if (event.relatedTarget !== null) {
				if (Prototype.E.descendantOf(event.relatedTarget,this.statusbarBox)) {
					return;
				}
			}
			if (!this.statusbarHover) { 
					return;
			}
		}
		this.statusbarHover = !this.statusbarHover;
		this.updateTickerFreezeState();
	},
});

Glydo.Statusbar.getSimpleCategoryFilter = function(category,max) {
	return ({
		condition: {
			contentType: category
		},
		orderBy: "-relevanceScore",
		limit: max ? max : 3
	});
};

Glydo.Statusbar.getAdCategoryFilter = function(category,max) {
	return ({
		condition: {
			contentType: category,
			contextRelation: "ad",
		},
		orderBy: "-relevanceScore",
		limit: max ? max : 3
	});
};

Glydo.Statusbar.PANE_GENERAL_ADS_FILTER = ({
	condition: {
		contentType: ["ad","product"],
		contextRelation: "ad",
	},
	orderBy: "-relevanceScore",
	limit: 20
});



Glydo.Statusbar.CATEGORIES = [
  {
  	group: "regular",
  	categories: [
		{
			name: "product_offers", 
			expectedWidth: 270,
			filter: {
				condition: {
					contentType: "product",
					contextItem: function(contextItem) {
						if (!contextItem) {
							return false;
						}
						return contextItem["@type"] == "product_offer";
					},
					contextRelation: "same_product_alternative_offer"
				},
				orderBy: "-relevanceScore",
				limit: 5
			},
			adFilter: null
		},
   		{
			name: "reviews", 
			expectedWidth: 284,
			filter: Glydo.Statusbar.getSimpleCategoryFilter("review")
		},
		{
			name: "news", 
			expectedWidth: 284,
			filter: Glydo.Statusbar.getSimpleCategoryFilter("news")
		},
		{
			name: "twitter",
			expectedWidth: 304,
			filter: Glydo.Statusbar.getSimpleCategoryFilter("twitter")
		},
		{
			name: "video",
			expectedWidth: 284,
			filter: Glydo.Statusbar.getSimpleCategoryFilter("video")
		},
		{ 
			name: "info",
			expectedWidth: 284,
			filter: Glydo.Statusbar.getSimpleCategoryFilter(["info"])
		},
		{
			name: "other",
			expectedWidth: 360,
			filter: Glydo.Statusbar.getSimpleCategoryFilter("other",4)
		},
		{
			name: "rss", 
			expectedWidth: 360,
			filter: Glydo.Statusbar.getSimpleCategoryFilter("rss",4)
		}
	]
  },
  {
  	group: "ads",
  	categories: [
  		{
  			name: "product",
  			filter: Glydo.Statusbar.getAdCategoryFilter("product",10),
  			dontShowCategoryButton: true,
  		},
  		{
  			name: "ad",
  			filter: Glydo.Statusbar.getAdCategoryFilter("ad",10),
  			dontShowCategoryButton: true,
  		}
  	]
  }
];
